import datetime
import pymongo
import logging
import urllib2
import hashlib

from config import MONGO_HOST, MONGO_PORT

STAT_URL = "https://store.juju.ubuntu.com/stats/counter"


log = logging.getLogger("charm.stat")


def _new_stat(charm, stat_id):
    # pre allocate document to avoid incremental resizing / ie mongo padding.
    stat = {"_id": stat_id}
    stat['days'] = dict.fromkeys(map(str, range(1, 366)), 0)
    stat['delta_days'] = dict.fromkeys(map(str, range(1, 366)), 0)
    stat['months'] = dict.fromkeys(map(str, range(1, 13)), 0)
    stat['store_key'] = charm['store_url'].rsplit('-', 1)[0]
    stat['branch_spec'] = charm['branch_spec']

    return stat


def _stat_url(charm, stat_type):
    if (charm['owner'] == 'charmers' and
        not charm['store_url'].startswith('cs:~charmers')):
        return "%s/%s:%s:%s" % (
            STAT_URL,
            stat_type,
            charm['series'],
            charm['name'])

    return "%s/%s:%s:%s:%s" % (
        STAT_URL,
        stat_type,
        charm['series'],
        charm['name'],
        charm['owner'])


def _get_stat(url):
    try:
        fh = urllib2.urlopen(url)
        dl = int(fh.read().strip())
        fh.close()
        return dl
    except:
        return None


def _stat_key(stat_type, spec, year):
    return hashlib.md5('%s-%s-%s' % (stat_type, year, spec)).hexdigest()


def gather(charms, stats, now):
    day = now.timetuple().tm_yday

    store_charms = charms.find({'store_url': {'$exists': True}})

    counter = 0
    total = store_charms.count()
    log.info("Store charms %d" % store_charms.count())

    month_day_start = day - now.day + 1

    for sc in store_charms:
        sc_id = _stat_key("charm-bundle", now.year, sc['branch_spec'])

        cs = stats.find_one({"_id": sc_id})
        if cs is None:
            cs = _new_stat(sc, sc_id)
            cs['year'] = now.year

        stat_url = _stat_url(sc, "charm-bundle")

        dl = _get_stat(stat_url)
        if dl is None:
            log.error("Error on stat %s" % stat_url)
            continue

        log.debug("%d -> %s -> %s" % (
            dl, sc['store_url'], stat_url))

        cs['days'][str(day)] = dl

        # play nice on a new year, including leap years
        prev_day_dl = dl
        if day == 1:
            prev_day_stat = stats.find_one({
                "_id": _stat_key(
                    'charm-bundle', now.year - 1, sc['branch_spec'])})
            if prev_day_stat:
                prev_day_dl = prev_day_stat['days'][
                    str((now - datetime.timedelta(1)).timetuple().tm_yday)]

        cs['delta_days'][str(day)] = dl - cs['days'].get(
            str(day - 1), prev_day_dl)
        cs['months'][str(now.month)] = sum([
            cs['delta_days'][str(x)] for x in range(month_day_start, day + 1)])
        cs['total'] = dl
        stats.save(cs)

        counter += 1
        if counter % 100 == 0:
            log.info("Processed %s/%s %s%%" % (
                counter, total, 100 * float(counter) / total))


def main():
    logging.basicConfig(
        level=logging.INFO,
        format="%(asctime)s: %(name)s@%(levelname)s: %(message)s")
    conn = pymongo.Connection(MONGO_HOST, MONGO_PORT)

    charms = conn.juju.charms
    stats = conn.juju.charm_stats
    now = datetime.datetime.now()
    gather(charms, stats, now)

if __name__ == '__main__':
    main()
