"""

Capture test artifacts from jenkins for a charm.


 - Get artifact files
 - Update charm info with last test run by provider.

"""
import gridfs
import json
import logging
import pymongo
import redis
import urllib2

from urllib2 import URLError

from config import (
    JENKINS_IN_QUEUE, JENKINS_OUT_QUEUE,
    MONGO_HOST, MONGO_PORT)

log = logging.getLogger("charm.jenkins")

JENKINS_QA_URL = (
    "https://jenkins.qa.ubuntu.com/job/"
    "%(series)s-%(provider)s-charm-%(charm)s/lastBuild/api/json")

JENKINS_ARTIFACT_URL = (
    "https://jenkins.qa.ubuntu.com/job/"
    "%(series)s-%(provider)s-charm-%(charm)s/%(build)d/artifact/%(artifact)s")


PROVIDERS = ['ec2', 'openstack', 'local']


def check_jenkins(db, fs, charm):

    if not charm['branch_spec'].startswith('~charmers'):
        return

    charm['tests'] = {}
    charm['test_results'] = {}

    for p in PROVIDERS:
        try:
            result_id, status = store_provider_results(db, fs, p, charm)
            if result_id is None:
                continue
            charm['tests'][p] = status
            charm['test_results'][p] = result_id
        except:
            log.exception("Unknown error while processing %s %s",
                          charm['branch_spec'], p)


def store_provider_results(db, fs, provider, charm):
    charm_result_url = JENKINS_QA_URL % (
        dict(series=charm['series'],
             provider=provider,
             charm=charm['name']))

    log.debug("Loading %s from %s", charm['name'], charm_result_url)

    try:
        contents = urllib2.urlopen(charm_result_url).read()
    except URLError:
        log.debug("No test result for %s @ %s", charm['branch_spec'], provider)
        return None, None

    result = json.loads(contents)

    # If we already have results no pointing in refetching.
    result_id = "%s::%s-%s" % (
        charm['branch_spec'], provider, result['number'])
    db_result = db.jenkins.find_one({'_id': result_id})
    if db_result is not None:
        return result_id, db_result['result']

    # Fetch test artifacts.
    artifacts = _fetch_artifacts(provider, charm, result)

    # Inject test metadata.
    result['branch_spec'] = charm['branch_spec']
    result['provider'] = provider
    result['artifacts'] = artifacts
    result['_id'] = result_id
    db.jenkins.insert(result)
    return (result_id, result['result'])


def _fetch_artifacts(provider, charm, result):
    artifacts = []
    for artifact in result['artifacts']:
        a_path = "%s/%s/%s/%s" % (
            charm['branch_spec'],
            provider,
            result['number'],
            artifact['displayPath'])
        a_url = JENKINS_ARTIFACT_URL % (dict(
            series=charm['series'],
            provider=provider,
            charm=charm['name'],
            build=result['number'],
            artifact=artifact['relativePath']))

        ## File sniffing

        # Load up the charm revision as a result property.
        if artifact['displayPath'] == 'charm-revision':
            charm_revision = urllib2.urlopen(a_url).read().strip()
            if not charm_revision:
                continue
            result['revno'] = int(charm_revision)
            continue

        # Mark the test result as graph runner enabled.
        if "graph-tests" in artifact['displayPath']:
            result['charmrunner'] = True

        # Skip the actual charm content
        if "charm-%s.zip" % charm['name'] == artifact['displayPath']:
            continue

        # XXX Short circuit before actual fetching.
        continue

        a_file = urllib2.urlopen(a_url)
        file_id = fs.put(a_file, path=a_path)
        artifact['file_id'] = file_id
        artifacts.append(artifact)

    return artifacts


def run(db, fs, queue,
        in_queue=JENKINS_IN_QUEUE,
        out_queue=JENKINS_OUT_QUEUE):

    while 1:
        item = queue.lpop(in_queue)
        if not item:
            return

        try:
            charm_data = json.loads(item)
            log.debug("Checking tests for %s", charm_data["branch_spec"])
            check_jenkins(db, fs, charm_data)
        except Exception, e:
            log.exception("jenkins error on %s: %s", item, str(e))
            continue
        else:
            queue.lpush(out_queue, json.dumps(charm_data))


if __name__ == '__main__':
    logging.basicConfig(
        level=logging.DEBUG,
        format="%(asctime)s: %(name)s@%(levelname)s: %(message)s")
    queue = redis.Redis()
    db = pymongo.Connection(MONGO_HOST, MONGO_PORT)['juju']
    fs = gridfs.GridFS(db)
    run(db, fs, queue)
