import contextlib
import json
import logging
import os
import redis
import sys

import config

log = logging.getLogger("charm.proof")


@contextlib.contextmanager
def get_proof_lib(new_path):
    if new_path not in sys.path:
        sys.path.append(new_path)
    try:
        import lib.proof as prooflib
        yield prooflib
    except ImportError:
        yield None
    finally:
        if new_path in sys.path:
            sys.path.remove(new_path)


def proof_charm(charm, prooflib):
    proof = {}
    lint, exit_code = prooflib.run(charm['branch_dir'])
    for line in lint:
        if not ':' in line:
            continue
        level, msg = line.split(':', 1)
        if level == "W" and 'name' in msg:
            continue
        proof.setdefault(level.lower(), []).append(msg)
    charm['proof'] = proof


def run(server, in_queue=config.PROOF_IN_QUEUE,
        out_queue=config.PROOF_OUT_QUEUE):
    if not os.path.isdir(config.CHARM_PROOF_PATH):
        err_msg = ("proof error before processing began: could not find "
                   "charm proof path.")
        log.exception(err_msg)
        log.exception("CHARM_PROOF_PATH: %s", config.CHARM_PROOF_PATH)
        log.exception("proof aborted.")
        return
    with get_proof_lib(config.CHARM_PROOF_PATH) as prooflib:
        if not prooflib:
            err_msg = ("proof error before processing began: could not "
                       "import charm proof lib.")
            log.exception(err_msg)
            log.exception("CHARM_PROOF_PATH: %s", config.CHARM_PROOF_PATH)
            log.exception("proof aborted.")
            return
        while 1:
            item = server.lpop(in_queue)
            if not item:
                return
            try:
                charm_data = json.loads(item)
                log.info("Proofing charm %s", charm_data["branch_spec"])
                proof_charm(charm_data, prooflib)
            except Exception as e:
                err = "proof error on %s: %s" % (
                      charm_data['branch_spec'], str(e))
                charm_data['proof'] = {'e': [err]}
                log.exception(err)
            server.lpush(out_queue, json.dumps(charm_data))


if __name__ == '__main__':
    logging.basicConfig(
        level=logging.WARNING,
        format="%(asctime)s: %(name)s@%(levelname)s: %(message)s")
    server = redis.Redis()
    run(server)
