import itertools

from dateutil.parser import parse

from charmworld.jobs.cstat import gather, _stat_url, _new_stat, _stat_key
from charmworld.testing import RemoteURLBase


class CharmStatTest(RemoteURLBase):

    url_prefix = "https://store.juju.ubuntu.com"
    stat_prefix = "stats/counter"

    def setUp(self):
        super(CharmStatTest, self).setUp()

    def assert_stat(self, stat, data):
        if 'months' in data:
            months = dict(zip(map(str, range(1, 13)), itertools.repeat(0)))
            months.update(data['months'])
            self.assertEqual(stat['months'], months)

        if 'days' in data:
            days = dict(zip(map(str, range(1, 366)), itertools.repeat(0)))
            days.update(data['days'])
            self.assertEqual(stat['days'], days)

        if 'delta_days' in data:
            delta = dict(zip(map(str, range(1, 366)), itertools.repeat(0)))
            delta.update(data['delta_days'])
            self.assertEqual(stat['delta_days'], delta)

        if 'total' in data:
            self.assertEqual(stat['total'], data['total'])

        if 'branch_spec' in data:
            self.assertEqual(stat['branch_spec'], data['branch_spec'])

    def test_new_stat_doc(self):
        charm = {
            'store_url': 'cs:precise/foobar-1',
            'branch_spec': '~charmers/charms/precise/foobar/trunk'}
        doc = _new_stat(charm, "unique")
        self.assertEqual(len(doc['days']), 365)
        self.assertEqual(len(doc['delta_days']), 365)
        self.assertEqual(len(doc['months']), 12)
        self.assertEqual(doc['branch_spec'], charm['branch_spec'])
        self.assertEqual(doc['store_key'],
                         charm['store_url'].rsplit('-', 1)[0])

    def test_stat_url(self):
        url_base = "%s/%s" % (self.url_prefix, self.stat_prefix)

        charm = {'owner': 'charmers', 'series': 'precise', 'name': 'foobar',
                 'store_url': 'cs:precise/foobar-1'}

        self.assertEqual(
            _stat_url(charm, "charm-bundle"),
            "%s/%s:%s" % (url_base, "charm-bundle", "precise:foobar"))

        self.assertEqual(
            _stat_url(charm, "charm-info"),
            "%s/%s:%s" % (url_base, "charm-info", "precise:foobar"))

        charm = {'owner': 'charmers', 'series': 'precise', 'name': 'foobar',
                 'store_url': 'cs:~charmers/precise/foobar-1'}

        self.assertEqual(
            _stat_url(charm, "charm-bundle"),
            "%s/%s:%s" % (
                url_base, "charm-bundle", "precise:foobar:charmers"))

        charm = {'owner': 'mike', 'series': 'precise', 'name': 'foobar',
                 'store_url': 'cs:~mike/precise/foobar-1'}

        self.assertEqual(
            _stat_url(charm, "charm-bundle"),
            "%s/%s:%s" % (
                url_base, "charm-bundle", "precise:foobar:mike"))

    def test_gather_charm_bundle(self):
        charm = {'owner': 'charmers', 'series': 'precise', 'name': 'foobar',
                 'store_url': 'cs:precise/foobar-1',
                 'branch_spec': '~charmers/charms/precise/foobar/trunk'}
        self.db.charms.insert(charm)

        self.url_result_map = {
            '/stats/counter/charm-bundle:precise:foobar': '42'}
        date = parse('2012/01/15')
        self.invoke(gather, self.db.charms, self.db.charm_stats, date)
        stat = self.db.charm_stats.find_one()
        self.assert_stat(
            stat,
            {'months': {'1': 42},
             'days': {'15': 42},
             'delta_days': {'15': 42},
             'total': 42,
             'branch_spec': '~charmers/charms/precise/foobar/trunk'})

        self.url_result_map = {
            '/stats/counter/charm-bundle:precise:foobar': '72'}
        date = parse('2012/01/16')
        self.invoke(gather, self.db.charms, self.db.charm_stats, date)
        stat = self.db.charm_stats.find_one()

        self.assert_stat(
            stat,
            {'months': {str(date.month): 72},
             'days': {'15': 42, '16': 72},
             'delta_days': {'15': 42, '16': 30}})

    def test_new_year(self):
        """Delta assessment is correct on new year (including leap years).
        """
        charm = {'owner': 'charmers', 'series': 'precise', 'name': 'foobar',
                 'store_url': 'cs:precise/foobar-1',
                 'branch_spec': '~charmers/charms/precise/foobar/trunk'}
        self.db.charms.insert(charm)
        self.url_result_map = {
            '/stats/counter/charm-bundle:precise:foobar': '42'}
        date = parse('2012/12/31')
        self.invoke(gather, self.db.charms, self.db.charm_stats, date)

        self.url_result_map = {
            '/stats/counter/charm-bundle:precise:foobar': '72'}
        date = parse('2013/1/1')
        self.invoke(gather, self.db.charms, self.db.charm_stats, date)
        stat = self.db.charm_stats.find_one(
            {"_id": _stat_key(
                'charm-bundle', date.year, charm['branch_spec'])})

        self.assert_stat(
            stat,
            {'months': {'1': 30},
             'days': {'1': 72},
             'delta_days': {'1': 30},
             'total': 72,
             'branch_spec': '~charmers/charms/precise/foobar/trunk'})

    def test_multiple_run_same_day(self):
        """Verify accidentally running the cron twice on the same day is okay.
        """
        charm = {'owner': 'charmers', 'series': 'precise', 'name': 'foobar',
                 'store_url': 'cs:precise/foobar-1',
                 'branch_spec': '~charmers/charms/precise/foobar/trunk'}
        self.db.charms.insert(charm)

        self.url_result_map = {
            '/stats/counter/charm-bundle:precise:foobar': '42'}
        date = parse('2012/01/15')
        self.invoke(
            gather, self.db.charms, self.db.charm_stats, date)

        self.url_result_map = {
            '/stats/counter/charm-bundle:precise:foobar': '72'}
        self.invoke(gather, self.db.charms, self.db.charm_stats, date)
        stat = self.db.charm_stats.find_one()
        self.assert_stat(
            stat,
            {'months': {'1': 72},
             'days': {'15': 72},
             'delta_days': {'15': 72},
             'total': 72,
             'branch_spec': '~charmers/charms/precise/foobar/trunk'})
