import gridfs
import mock
import os

from charmworld.jobs import jenkins
from charmworld.testing import RemoteURLBase


DATA_DIR = os.path.join(os.path.dirname(__file__), "data")


def _data_file(url_frag, contents=None):
    if "/artifact/" in url_frag:
        _, fname = url_frag.rsplit("/", 1)
    else:
        fname, _ = url_frag.split("/", 1)

    if contents is None:
        with open(os.path.join(DATA_DIR, fname)) as fh:
            contents = fh.read()

    return (url_frag, contents)


class JenkinsTest(RemoteURLBase):

    url_prefix = "https://jenkins.qa.ubuntu.com/job/"

    def setUp(self):
        super(JenkinsTest, self).setUp()
        self.fs = gridfs.GridFS(self.db)

    def test_jenkins(self):
        charm = {'branch_spec': '~charmers'}

        with mock.patch("charmworld.jobs.jenkins.store_provider_results") as s:
            s.side_effect = [(1, "FAIL"), (None, None), KeyError("xz")]
            jenkins.check_jenkins(self.db, self.fs, charm)
        self.assertEqual(
            charm['tests'], {"ec2": "FAIL"})

        self.assertEqual(
            charm['test_results'], {"ec2": 1})

        charm = {'branch_spec': ''}
        jenkins.check_jenkins(self.db, self.fs, charm)
        self.assertEqual(charm, {'branch_spec': ''})

    def test_store_results(self):
        self.url_result_map = dict([
            _data_file(
                "precise-openstack-charm-mysql/lastBuild/api/json"),
            _data_file(
                ("precise-openstack-charm-mysql/31/artifact/"
                 "charm-graph-tests-logs.tar.gz"), "def"),
            _data_file(
                ("precise-openstack-charm-mysql/31/artifact/"
                 "charm-graph-tests-plans.zip"), "xyz"),
            _data_file(
                ("precise-openstack-charm-mysql/31/artifact/"
                 "charm-unit-tests-logs.tar.gz"), "xyz"),
            _data_file(
                ("precise-openstack-charm-mysql/31/artifact/"
                 "charm-revision"), "21\n\n"),
            _data_file(
                ("precise-openstack-charm-mysql/31/artifact/"
                 "charm-mysql.zip"), "abc"),
            # This file's existence seems to be some jenkins
            # environment cleaning issue.
            _data_file(
                ("precise-openstack-charm-mysql/31/artifact/"
                 "charms/precise/lodgeit/opt/lodgeit.tar.gz"), "abc")])

        result = self.invoke(
            jenkins.store_provider_results,
            self.db, self.fs, "openstack",
            {"name": "mysql",
             "series": "precise",
             "branch_spec": "~charmers/precise/mysql/trunk"})
        test_run = self.db.jenkins.find_one()
        self.assertEqual(test_run['revno'], 21)
        self.assertEqual(
            test_run['branch_spec'], '~charmers/precise/mysql/trunk')

        # DISABLED
        self.assertEqual(len(test_run['artifacts']), 0)
        self.assertEqual(test_run['charmrunner'], True)
        self.assertEqual(test_run['_id'], result[0])
        self.assertEqual(test_run['result'], result[1])

        ## DISABLED
        # fetch one of the files to verify
        #fh = self.fs.get(test_run['artifacts'][0]['file_id'])
        #self.assertEqual(fh.read(), "def")

    def test_store_results_exists(self):
        self.url_result_map = dict([
            _data_file("precise-openstack-charm-mysql/lastBuild/api/json")])
        rid = "~charmers/precise/mysql/trunk::openstack-31"
        self.db.jenkins.insert({"_id": rid, "result": "SUCCESS"})

        result = self.invoke(
            jenkins.store_provider_results,
            self.db, self.fs, "openstack",
            {"name": "mysql",
             "series": "precise",
             "branch_spec": "~charmers/precise/mysql/trunk"})
        self.assertEqual(result, (rid, "SUCCESS"))

    def test_store_results_error(self):
        result = self.invoke(
            jenkins.store_provider_results,
            self.db, self.fs, "openstack",
            {"name": "mysql",
             "series": "precise",
             "branch_spec": "~charmers/precise/mysql/trunk"})
        self.assertEqual(result, (None, None))
