import json
import logging
from mock import patch
import redis
from unittest import TestCase

from charmworld.jobs import config
from charmworld.jobs.proof import run


def bad_proofer(*args, **kwargs):
    raise Exception('Forced error from mock')


class ProofTest(TestCase):

    def setUp(self):
        super(ProofTest, self).setUp()
        self.in_queue = 'test-charm-proof'
        self.out_queue = 'test-charm-test'
        self.server = redis.Redis()

        self.server.lpush(self.in_queue, json.dumps(
            {'branch_dir': 'foo', 'branch_spec': 'bar'}))

    def tearDown(self):
        super(ProofTest, self).tearDown()
        self.server.delete(self.in_queue)
        self.server.delete(self.out_queue)

    @patch('charmworld.jobs.proof.proof_charm', new=bad_proofer)
    def test_proofing_failure_added_as_lint(self):
        run(self.server, in_queue=self.in_queue, out_queue=self.out_queue)
        item = self.server.lpop(self.out_queue)
        charm = json.loads(item)
        err = "proof error on bar: Forced error from mock"
        self.assertEqual({'e': [err]}, charm['proof'])

    def test_charm_proof_library_not_found(self):
        # If the charm-proof library cannot be found, an error is logged
        # and the proof loop aborted.
        proof_logger = logging.getLogger("charm.proof")
        handler = logging.handlers.BufferingHandler(20)
        proof_logger.addHandler(handler)
        real_proof_path = config.CHARM_PROOF_PATH

        def cleanup():
            proof_logger.removeHandler(handler)
            config.CHARM_PROOF_PATH = real_proof_path

        self.addCleanup(cleanup)
        config.CHARM_PROOF_PATH = '/does/not/exist'
        run(self.server, in_queue=self.in_queue, out_queue=self.out_queue)
        log_messages = [record.getMessage() for record in handler.buffer]
        err = ("proof error before processing began: "
               "could not find charm proof path.")
        self.assertIn(err, log_messages)
        self.assertIn('CHARM_PROOF_PATH: /does/not/exist', log_messages)
        self.assertIn('proof aborted.', log_messages)
