import pymongo


def getconnection(settings, fsync=False, safe=False):
    """Initiliaze the PyMongo database Connection object.

    :param settings: dict of application settings including the mongo.X info.

    """
    connection = pymongo.Connection(
        settings.get('mongo.host'),
        int(settings.get('mongo.port')),
        fsync=fsync,
        safe=safe,
    )
    return connection


def getdb(connection, database_name):
    """Helper to access the db instance.

    This is needed because the test runner nose loads views which tried to
    import db directly, before the main() is run and initialize_db is run with
    the connection information.

    :param connection: pymongo connection object.
    :param database_name: mongo db name to connect to.
    """
    return getattr(connection, database_name)


def parse_sso_fields(payload):
    """Process the data form a sso request to a normal clean dict."""
    user_sso_data = set([
        'openid.sreg.nickname',
        'openid.sreg.fullname',
        'openid.sreg.email',
        'openid.sreg.timezone',
    ])
    common = user_sso_data.intersection(payload)
    # We're using the last bit of the key in our User object.
    user_data = dict((key.split('.')[2], payload[key]) for key in common)
    groups = []
    if payload['openid.lp.is_member'] != '':
        groups = payload['openid.lp.is_member'].split(',')
    user_data['groups'] = groups
    return user_data


class UserMgr(object):
    """Handle non-instance user actions."""

    @staticmethod
    def auth_principalfinder(userid, request):
        """Pyramid wants to know what principals are associated with a user.

        This is a helper provided for the RequestWithUser factory.
        """
        user = request.user
        if user is None:
            return None
        group_principals = ['group %s' % group for group in user.groups]
        return ['user %s' % user.userid] + group_principals

    @staticmethod
    def find_one(db, userid):
        found = db.users.find_one({'userid': userid})

        if not found:
            return None
        else:
            # Make sure we return a instantiated object vs the raw dictionary
            # from mongo.
            userid = found['userid']
            del(found['userid'])
            return User(userid, **found)

    @staticmethod
    def from_sso(payload):
        """Generate a User object instance from the SSO payload.

        We take the SSO dict and turn it into something that a User object,
        without being SSO aware, can easily handle and process.

        """
        user_data = parse_sso_fields(payload)
        user_id = payload['openid.claimed_id']
        return User(user_id, **user_data)


class User(object):
    """A user in our system."""

    # Attributes are defined as a list so we can easily create a
    # dict(someuser) helper to dump to mongodb.
    attributes = [
        '_id',
        'userid',
        'nickname',
        'fullname',
        'email',
        'groups',
        'timezone',
    ]

    def __init__(self, userid, **kwargs):
        self.userid = userid

        # Set some sane defaults since this depends on the users providing us
        # with the data.
        self.nickname = kwargs.get('nickname', 'Not Provided')
        self.fullname = kwargs.get('fullname', 'Not Provided')
        self.email = kwargs.get('email', None)
        self.groups = kwargs['groups']
        self.timezone = kwargs.get('timezone', 'GMT')

        # We need to worry about if this is a mongodb record or just a new
        # user object. A new object has no mongo id.
        if '_id' in kwargs:
            self._id = kwargs['_id']
        else:
            self._id = None

    def __iter__(self):
        """Allow the User to be dumped to a dict for mongo."""
        for key in self.attributes:
            yield(key, getattr(self, key))

    def in_group(self, group):
        return True if group in self.groups else False

    def save(self, db):
        """Actual storing of the instance into mongodb."""
        mongo_data = dict(self)
        if not self._id:
            db.users.save(mongo_data)
        else:
            self._id = db.users.insert(mongo_data)

    def update_from_sso(self, payload):
        """Update the stored user data from a fresh sso payload dump."""
        user_data = parse_sso_fields(payload)
        for key, val in user_data.items():
            setattr(self, key, val)


class QAData(object):
    """Build a usable object out of the QA questions and instance data."""

    def _load_questionset(self):
        """Load the qa questions we want to have data about."""
        stored_questions = self.db.qa.find()
        # We index them for easy fetching of each categories data later.
        self.qa_categories = dict([(q['name'], q) for q in stored_questions])

    def __init__(self, db, charm_data=None):
        self.db = db
        self._load_questionset()
        self.charm_data = charm_data

    @property
    def categories(self):
        return sorted(self.qa_categories)

    @property
    def score(self):
        """Calculate the score for whole qa set across categories."""

        # If we have no qa data then we need to respond with not available
        if not self.charm_data:
            return None

        score = 0
        for category in self.categories:
            category_score = self.category_score(category)
            if category_score is not None:
                score = score + category_score

        return score

    def category_score(self, category):
        """Given a category retrieve the scores for it."""
        score = 0
        if self.charm_data is None or category not in self.charm_data:
            return None

        category_questions = self.qa_categories[category]['questions']
        charm_category_scores = self.charm_data[category]

        for qnum, question in enumerate(category_questions):
            score_key = "{0}_{1}".format(category, qnum)
            q_score = charm_category_scores.get(score_key, 0)
            # The score could be '' if it's unknown vs yes/no
            if q_score:
                score = score + int(q_score)
        return score

    def category_max_score(self, category):
        """How many possible points does the category have?"""
        return len(self.qa_categories[category]['questions'])
