from ConfigParser import ConfigParser
from cStringIO import StringIO
from datetime import datetime
from datetime import timedelta
from os import environ, unlink
import tempfile
from textwrap import dedent
from unittest import TestCase

from charmworld.utils import get_ini
from charmworld.utils import pretty_timedelta
from charmworld.lib.config import merge_configs


class TestPrettyTimeDeltaFormatter(TestCase):

    def test_months_apart_multiple_dates(self):
        """We can pass multiple dates in to get the diff."""
        end = datetime(year=2012, month=12, day=1)
        start = datetime(year=2012, month=3, day=1)
        pretty = pretty_timedelta(start, end)

        self.assertEqual('9 months', pretty)

    def test_bad_date_order(self):
        """If we pass in the start date first we get an error."""
        start = datetime(year=2012, month=12, day=1)
        end = datetime(year=2012, month=12, day=2)
        with self.assertRaises(ValueError):
            pretty_timedelta(end, start)

    def test_days_apart(self):
        """A single delta will get us the correct days apart."""
        delta = datetime.now() - timedelta(10)
        pretty = pretty_timedelta(delta)
        self.assertEqual('10 days', pretty)


class TestIniHelper(TestCase):

    def tearDown(self):
        # Make sure we clear out any INI setting from the environ
        if 'INI' in environ:
            del environ['INI']

    def test_default_charmworld_ini(self):
        """By default we get a dict of our charmworld.ini file."""
        ini = get_ini()
        self.assertIn('project_name', ini)
        self.assertIn('http_cache', ini)

    def test_overridden_ini(self):
        """Setting the ENV value we can override the ini read."""
        environ['INI'] = 'test.ini'
        ini = get_ini()
        self.assertIn('project_name', ini)
        self.assertEqual('Test Charm Browser', ini.get('project_name'))


class TestConfigSetup(TestCase):

    def makeDefaultIniFile(self, filename):
        with open(filename, 'w') as default_ini_file:
            default_ini_file.write(dedent("""
                [section_one]
                foo = one
                [section_two]
                bar = two
            """))

    def getTempFileName(self):
        result = tempfile.mktemp()
        self.addCleanup(unlink, result)
        return result

    def test_no_custom_ini_file(self):
        # If no custom .ini file exists, the contents of the default file
        # simply copied to the merged config file.
        default_ini_name = self.getTempFileName()
        result_ini_name = self.getTempFileName()
        self.makeDefaultIniFile(default_ini_name)
        merge_configs(
            result_ini_name, default_ini_name)
        result_config = ConfigParser()
        result_config.read(result_ini_name)
        self.assertEqual(
            set(('section_one', 'section_two')), set(result_config.sections()))
        self.assertEqual(['foo'], result_config.options('section_one'))
        self.assertEqual('one', result_config.get('section_one', 'foo'))
        self.assertEqual(['bar'], result_config.options('section_two'))
        self.assertEqual('two', result_config.get('section_two', 'bar'))

    def test_custom_ini_file(self):
        # If a custom .ini file exists, its settings override those from
        # the default .ini file.
        default_ini_name = self.getTempFileName()
        custom_ini_name = self.getTempFileName()
        result_ini_name = self.getTempFileName()
        with open(custom_ini_name, 'w') as custom_ini_file:
            custom_ini_file.write(dedent("""
                [section_one]
                foo = NINE
            """))
        self.makeDefaultIniFile(default_ini_name)
        merge_configs(
            result_ini_name, default_ini_name, custom_ini_name)
        result_config = ConfigParser()
        result_config.read(result_ini_name)
        self.assertEqual(
            set(('section_one', 'section_two')), set(result_config.sections()))
        self.assertEqual(['foo'], result_config.options('section_one'))
        self.assertEqual('NINE', result_config.get('section_one', 'foo'))
        self.assertEqual(['bar'], result_config.options('section_two'))
        self.assertEqual('two', result_config.get('section_two', 'bar'))

    def test_custom_ini_file_with_unexpected_data(self):
        # Definitions in the custom .ini file that do not have correspoding
        # definitions in the default .ini file are copied into the merged
        # .ini file, but a warning is printed.
        default_ini_name = self.getTempFileName()
        custom_ini_name = self.getTempFileName()
        result_ini_name = self.getTempFileName()
        with open(custom_ini_name, 'w') as custom_ini_file:
            custom_ini_file.write(dedent("""
                [section_one]
                foo = NINE
                baz = whatever
                [new_section]
                something = completely_different
            """))
        self.makeDefaultIniFile(default_ini_name)
        err_output = StringIO()
        merge_configs(
            result_ini_name, default_ini_name, custom_ini_name, err=err_output)
        result_config = ConfigParser()
        result_config.read(result_ini_name)
        self.assertEqual(
            set(('section_one', 'section_two', 'new_section')),
            set(result_config.sections()))
        self.assertEqual(['foo', 'baz'], result_config.options('section_one'))
        self.assertEqual('NINE', result_config.get('section_one', 'foo'))
        self.assertEqual('whatever', result_config.get('section_one', 'baz'))
        self.assertEqual(['bar'], result_config.options('section_two'))
        self.assertEqual('two', result_config.get('section_two', 'bar'))
        self.assertEqual(['something'], result_config.options('new_section'))
        self.assertEqual(
            'completely_different',
            result_config.get('new_section', 'something'))
        expected_warnings = (
            'Warning: %s defines sections that do not appear in %s: '
            'new_section\n'
            'Warning: Section section_one, option baz only defined in %s, '
            'not in %s\n') % (
                custom_ini_name, default_ini_name, custom_ini_name,
                default_ini_name)
        self.assertEqual(expected_warnings, err_output.getvalue())
