from deform import Form
from deform import ValidationFailure
import fnmatch
import json
import os
import pymongo
from pyramid.httpexceptions import HTTPNotFound
from pyramid.view import view_config
from webob import Response

from charmworld import cached_view_config
from charmworld.forms.qa_assessment import get_qa_form_schema
from charmworld.models import QAData
from charmworld.views import log
from charmworld.views import SANITIZE
from charmworld.views.helpers import format_change
from charmworld.views.helpers import format_proof
from charmworld.views.helpers import found
from charmworld.views.helpers import interfaces
from charmworld.views.helpers import name_filter
from charmworld.views.helpers import sub_filter


def _charm_readme(charm):
    # Find a readme
    dirents = os.listdir(charm['branch_dir'])
    dirents = dict([(k.lower(), v) for k, v in zip(dirents, dirents)])
    matches = fnmatch.filter(dirents.keys(), "readme*")
    if not matches:
        return [None, None]
    matches.sort(lambda x, y: cmp(len(x), len(y)))
    readme_path = os.path.join(charm['branch_dir'], dirents[matches[0]])
    readme_extension = os.path.splitext(readme_path)[1][1:]

    with open(readme_path) as fh:
        return [readme_extension, unicode(fh.read(), "utf8")]


def _charm_view(request, charm):
    # Compute display values for requires, provides, and config.
    charm["d_requires"] = d_requires = []

    # Defensive parsing, i know.. ingest time would have been nice to weed
    # these out, but we want to show/report things that are broken and flag
    # them appropriately.
    requires = charm.get('requires', {})
    if requires is None:
        log.warning("charm with invalid requires %s", charm['branch_spec'])
        requires = {}
    for k, v in requires.items():
        d = {"name": k, "interface": v["interface"]}
        d_requires.append(d)

    charm["d_provides"] = d_provides = []
    for k, v in charm.get("provides", {}).items():

        if isinstance(v, basestring):
            interface = v
        else:
            interface = v['interface']
        d = {"name": k, "interface": interface}
        d_provides.append(d)

    charm["d_config"] = d_config = []

    if charm.get('config', {'options': {}}).get('options'):
        for k, v in charm.get("config", {"options": {}})["options"].items():
            d = {"name": k,
                 "default": v.get('default', u''),
                 'type': v.get('type', u''),
                 'description': v.get('description', u'')}
            d_config.append(d)

    # Find the same package in other archives.
    others = list(request.db.charms.find(
        {"name": charm["name"], "series": charm["series"],
         "branch_spec": {"$ne": charm["branch_spec"]}}))

    readme_format, readme = _charm_readme(charm)
    charm['maintainer'] = name_filter(charm.get('maintainer', ''))
    return {
        "charm": charm,
        'format_change': format_change,
        'format_proof': format_proof,
        "name": charm["name"],
        "others": others,
        "project": request.registry.settings.get('project_name', 'Not Set.'),
        "qadata": QAData(request.db, charm.get('qa', None)),
        "readme_format": readme_format,
        "readme": readme,
    }


def _config_view(charm):
    return {"charm": charm, "onload": "prettyPrint()"}


def _hook_view(request, charm, hook):
    hook_path = os.path.join(charm["branch_dir"], "hooks", hook)
    data = {
        "project": request.registry.settings.get('project_name', 'Not Set.'),
        "hook_raw": "Hook not found!",
        "charm": charm,
        "hook": hook,
        "onload": "prettyPrint()"
    }

    if not os.path.exists(hook_path):
        log.warning("Hook not found %s %s %s" % (
            charm['branch_spec'],
            hook,
            hook_path))
        return data

    with open(hook_path) as fh:
        hook_raw = fh.read()
        data['hook_raw'] = hook_raw
    return data


def _json_charm(charm):
    charm['bzr_branch'] = "lp:%s" % charm['branch_spec']
    #print "store data", charm.get('store_data', {})
    #print "distro series", charm.get('distro_series', {})
    charm_keys = set(charm.keys())
    if charm.get('store_data', {}).get('revision') is not None:
        charm['store_revision'] = charm['store_data']['revision']

    for k in SANITIZE.intersection(charm_keys):
        del charm[k]
    return Response(
        json.dumps(charm, indent=2),
        headerlist=[
            ("Access-Control-Allow-Origin", "*"),
            ("Access-Control-Allow-Headers", "X-Requested-With")],
        content_type=u"application/json")


@cached_view_config(
    route_name="charm-collection",
    renderer="charmworld:templates/charm-collection.pt")
def charm_collection(request):
    query, sub_only = sub_filter({"owner": "charmers"}, request)
    charms = found(request.db.charms.find(
        query,
        {"name": 1, "owner": 1, "series": 1, "summary": 1, "short_url": 1},
        sort=[("name", pymongo.ASCENDING)]))
    return {"charms": list(charms), 'sub_only': sub_only}


@cached_view_config(
    route_name="personal-collection",
    renderer="charmworld:templates/charm-collection.pt")
def personal_collection(request):
    query, sub_only = sub_filter({
        "owner": request.matchdict["owner"]
    }, request)
    charms = found(request.db.charms.find(
        query,
        {"name": 1, "owner": 1, "series": 1, "summary": 1, "short_url": 1},
        sort=[("name", pymongo.ASCENDING)]))
    return {"charms": list(charms), "sub_only": sub_only}


@cached_view_config(
    route_name="series-collection",
    renderer="charmworld:templates/charm-collection.pt")
def series_collection(request):
    query, sub_only = sub_filter(
        {"owner": "charmers", "series": request.matchdict["series"]}, request)
    charms = found(request.db.charms.find(
        query,
        {
            "name": 1,
            "owner": 1,
            "series": 1,
            "summary": 1,
            "short_url": 1,
        },
        sort=[("name", pymongo.ASCENDING)]))
    return {"charms": list(charms), 'sub_only': sub_only}


@cached_view_config(
    route_name="personal-series",
    renderer="charmworld:templates/charm-collection.pt")
def personal_series_collection(request):
    query, sub_only = sub_filter(
        {
            "owner": request.matchdict["owner"],
            "series": request.matchdict["series"]
        },
        request)

    charms = found(request.db.charms.find(
        query,
        {"name": 1, "owner": 1, "series": 1, "summary": 1, "short_url": 1},
        sort=[("name", pymongo.ASCENDING)]))
    return {"charms": list(charms), 'sub_only': sub_only}


@cached_view_config(
    route_name="personal-charm",
    renderer="charmworld:templates/charm.pt")
def personal_charm(request):
    charm = found(request.db.charms.find_one(
        {"name": request.matchdict['charm'],
         "owner": request.matchdict['owner'],
         "series": request.matchdict['series']}))
    return _charm_view(request, charm)


@cached_view_config(
    route_name="personal-charm-revision-json")
@cached_view_config(
    route_name="personal-charm-json")
def person_charm_json(request):
    match = request.matchdict
    if 'revision' in match and not match['revision'].isdigit():
        match['charm'] = "%s-%s" % (
            match['charm'], match['revision'])
    charm = found(request.db.charms.find_one(
        {"name": match['charm'],
         "owner": match['owner'],
         "series": match['series']}))
    return _json_charm(charm)


@cached_view_config(
    route_name="charm",
    renderer="charmworld:templates/charm.pt")
def distro_charm(request):
    charm = found(request.db.charms.find_one(
        {"name": request.matchdict["charm"],
         "owner": "charmers",
         "series": request.matchdict["series"]}))
    return _charm_view(request, charm)


@cached_view_config(
    route_name="charm-revision-json",
)
@cached_view_config(
    route_name="charm-json",
)
def distro_charm_json(request):
    match = request.matchdict
    if 'revision' in match and not match['revision'].isdigit():
        match['charm'] = "%s-%s" % (
            match['charm'], match['revision'])
    charm = found(request.db.charms.find_one(
        {"name": match["charm"],
         "owner": "charmers",
         "series": match["series"]}))
    return _json_charm(charm)


@cached_view_config(
    route_name="hook",
    renderer="charmworld:templates/hook.pt")
def hook(request):
    charm = found(request.db.charms.find_one({
        "owner": "charmers",
        "series": request.matchdict["series"],
        "name": request.matchdict["charm"]}))
    return _hook_view(request, charm, request.matchdict["hook"])


@cached_view_config(
    route_name="personal-hook",
    renderer="charmworld:templates/hook.pt")
def personal_hook(request):
    charm = found(request.db.charms.find_one({
        "owner": request.matchdict["owner"],
        "series": request.matchdict["series"],
        "name": request.matchdict["charm"]}))
    return _hook_view(request, charm, request.matchdict["hook"])


@cached_view_config(
    route_name="config",
    renderer="charmworld:templates/config.pt")
def config(request):
    charm = found(request.db.charms.find_one({
        "owner": "charmers",
        "series": request.matchdict["series"],
        "name": request.matchdict["charm"]}))
    return _config_view(charm)


@cached_view_config(
    route_name="personal-config",
    renderer="charmworld:templates/config.pt")
def personal_config(request):
    charm = found(request.db.charms.find_one({
        "owner": request.matchdict["owner"],
        "series": request.matchdict["series"],
        "name": request.matchdict["charm"]}))
    return _config_view(charm)


@cached_view_config(
    route_name="interface-collection-map",
    renderer="charmworld:templates/interface-collection.pt")
@cached_view_config(
    route_name="interface-collection",
    renderer="charmworld:templates/interface-collection-redux.pt")
def interface_collection(request):
    return {
        "interfaces": interfaces(request.db),
        "project": request.registry.settings.get('project_name', 'Not Set.'),
    }


@cached_view_config(
    route_name="interface",
    renderer="charmworld:templates/interface.pt")
def interface(request):
    iface = request.matchdict["interface"]
    return {
        "interface": iface,
        "providers": list(request.db.charms.find({"i_provides": iface},
                          sort=[("name", pymongo.ASCENDING)])),
        "requirers": list(request.db.charms.find({"i_requires": iface},
                          sort=[("name", pymongo.ASCENDING)]))}


@view_config(
    route_name="charm-qa-edit",
    renderer='charmworld:templates/qa_edit.pt',
    permission='edit',
)
def quality_edit(request):
    valid_data = {}
    charm = request.db.charms.find_one({
        "name": request.matchdict["charm"],
        "owner": "charmers",
        "series": request.matchdict["series"],
    })

    if not charm:
        raise HTTPNotFound()

    qa_questions = request.db.qa.find()
    schema = get_qa_form_schema(qa_questions)
    qa_form = Form(schema, buttons=('submit',))
    qa_form.css_class = 'form-horizontal well'

    if 'submit' in request.POST:
        data = request.POST.items()
        try:
            valid_data = qa_form.validate(data)
            charm['qa'] = valid_data
            request.db.charms.save(charm)
            form = qa_form.render(valid_data)
        except ValidationFailure, exc:
            qa_form = exc
            valid_data = {}
            form = qa_form.render()
    elif 'qa' in charm:
        # There's existing data to use to fill the form with.
        valid_data = charm['qa']
        form = qa_form.render(valid_data)
    else:
        form = qa_form.render(valid_data)

    return {
        'charm': charm,
        'form': form,
        "values": valid_data,
    }
